@@ -4,19 +4,35 @@
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
+from __future__ import absolute_import
+
+import errno
+import os
+import re
+import socket
 
-import os, re, socket, errno
-from cStringIO import StringIO
-from mercurial import encoding, util
 from mercurial.i18n import _
+from mercurial import (
+    encoding,
+    error,
+    util,
+)
 
-from common import NoRepo, commit, converter_source, checktool
-from common import makedatetimestamp
-import cvsps
+from . import (
+    common,
+    cvsps,
+)
+
+stringio = util.stringio
+checktool = common.checktool
+commit = common.commit
+converter_source = common.converter_source
+makedatetimestamp = common.makedatetimestamp
+NoRepo = common.NoRepo
 
 class convert_cvs(converter_source):
-    def __init__(self, ui, path, rev=None):
-        super(convert_cvs, self).__init__(ui, path, rev=rev)
+    def __init__(self, ui, path, revs=None):
+        super(convert_cvs, self).__init__(ui, path, revs=revs)
 
         cvs = os.path.join(path, "CVS")
         if not os.path.exists(cvs):
@@ -41,14 +57,17 @@ class convert_cvs(converter_source):
         self.changeset = {}
 
         maxrev = 0
-        if self.rev:
+        if self.revs:
+            if len(self.revs) > 1:
+                raise error.Abort(_('cvs source does not support specifying '
+                                   'multiple revs'))
             # TODO: handle tags
             try:
                 # patchset number?
-                maxrev = int(self.rev)
+                maxrev = int(self.revs[0])
             except ValueError:
-                raise util.Abort(_('revision %s is not a patchset number')
-                                 % self.rev)
+                raise error.Abort(_('revision %s is not a patchset number')
+                                 % self.revs[0])
 
         d = os.getcwd()
         try:
@@ -136,7 +155,7 @@ class convert_cvs(converter_source):
                                 passw = part2
                                 break
                         pf.close()
-                    except IOError, inst:
+                    except IOError as inst:
                         if inst.errno != errno.ENOENT:
                             if not getattr(inst, 'filename', None):
                                 inst.filename = cvspass
@@ -147,7 +166,7 @@ class convert_cvs(converter_source):
                 sck.send("\n".join(["BEGIN AUTH REQUEST", root, user, passw,
                                     "END AUTH REQUEST", ""]))
                 if sck.recv(128) != "I LOVE YOU\n":
-                    raise util.Abort(_("CVS pserver authentication failed"))
+                    raise error.Abort(_("CVS pserver authentication failed"))
 
                 self.writep = self.readp = sck.makefile('r+')
 
@@ -190,7 +209,7 @@ class convert_cvs(converter_source):
         self.writep.flush()
         r = self.readp.readline()
         if not r.startswith("Valid-requests"):
-            raise util.Abort(_('unexpected response from CVS server '
+            raise error.Abort(_('unexpected response from CVS server '
                                '(expected "Valid-requests", but got %r)')
                              % r)
         if "UseUnchanged" in r:
@@ -208,11 +227,11 @@ class convert_cvs(converter_source):
             # file-objects returned by socket.makefile() do not handle
             # large read() requests very well.
             chunksize = 65536
-            output = StringIO()
+            output = stringio()
             while count > 0:
                 data = fp.read(min(count, chunksize))
                 if not data:
-                    raise util.Abort(_("%d bytes missing from remote file")
+                    raise error.Abort(_("%d bytes missing from remote file")
                                      % count)
                 count -= len(data)
                 output.write(data)
@@ -249,18 +268,18 @@ class convert_cvs(converter_source):
             else:
                 if line == "ok\n":
                     if mode is None:
-                        raise util.Abort(_('malformed response from CVS'))
+                        raise error.Abort(_('malformed response from CVS'))
                     return (data, "x" in mode and "x" or "")
                 elif line.startswith("E "):
                     self.ui.warn(_("cvs server: %s\n") % line[2:])
                 elif line.startswith("Remove"):
                     self.readp.readline()
                 else:
-                    raise util.Abort(_("unknown CVS response: %s") % line)
+                    raise error.Abort(_("unknown CVS response: %s") % line)
 
     def getchanges(self, rev, full):
         if full:
-            raise util.Abort(_("convert from cvs do not support --full"))
+            raise error.Abort(_("convert from cvs does not support --full"))
         self._parse()
         return sorted(self.files[rev].iteritems()), {}, set()
 
