@@ -149,6 +149,8 @@ def topicmatch(ui, kw):
     for name, docs in itertools.chain(
         extensions.enabled(False).iteritems(),
         extensions.disabled().iteritems()):
+        if not docs:
+            continue
         mod = extensions.load(ui, name, '')
         name = name.rpartition('.')[-1]
         if lowercontains(name) or lowercontains(docs):
@@ -182,12 +184,16 @@ def loaddoc(topic, subdir=None):
     return loader
 
 internalstable = sorted([
-    (['bundles'], _('container for exchange of repository data'),
+    (['bundles'], _('Bundles'),
      loaddoc('bundles', subdir='internals')),
-    (['changegroups'], _('representation of revlog data'),
+    (['changegroups'], _('Changegroups'),
      loaddoc('changegroups', subdir='internals')),
-    (['revlogs'], _('revision storage mechanism'),
+    (['requirements'], _('Repository Requirements'),
+     loaddoc('requirements', subdir='internals')),
+    (['revlogs'], _('Revision Logs'),
      loaddoc('revlogs', subdir='internals')),
+    (['wireprotocol'], _('Wire Protocol'),
+     loaddoc('wireprotocol', subdir='internals')),
 ])
 
 def internalshelp(ui):
@@ -332,10 +338,13 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
         if not doc:
             doc = _("(no help text available)")
         if util.safehasattr(entry[0], 'definition'):  # aliased command
+            source = entry[0].source
             if entry[0].definition.startswith('!'):  # shell alias
-                doc = _('shell alias for::\n\n    %s') % entry[0].definition[1:]
+                doc = (_('shell alias for::\n\n    %s\n\ndefined by: %s\n') %
+                       (entry[0].definition[1:], source))
             else:
-                doc = _('alias for: hg %s\n\n%s') % (entry[0].definition, doc)
+                doc = (_('alias for: hg %s\n\n%s\n\ndefined by: %s\n') %
+                       (entry[0].definition, doc, source))
         doc = doc.splitlines(True)
         if ui.quiet or not full:
             rst.append(doc[0])
@@ -349,8 +358,8 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
             mod = extensions.find(name)
             doc = gettext(mod.__doc__) or ''
             if '\n' in doc.strip():
-                msg = _('(use "hg help -e %s" to show help for '
-                        'the %s extension)') % (name, name)
+                msg = _("(use 'hg help -e %s' to show help for "
+                        "the %s extension)") % (name, name)
                 rst.append('\n%s\n' % msg)
         except KeyError:
             pass
@@ -365,7 +374,7 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
 
         if not ui.verbose:
             if not full:
-                rst.append(_('\n(use "hg %s -h" to show more help)\n')
+                rst.append(_("\n(use 'hg %s -h' to show more help)\n")
                            % name)
             elif not ui.quiet:
                 rst.append(_('\n(some details hidden, use --verbose '
@@ -441,21 +450,21 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
             rst.append('\n%s\n' % optrst(_("global options"),
                                          commands.globalopts, ui.verbose))
             if name == 'shortlist':
-                rst.append(_('\n(use "hg help" for the full list '
-                             'of commands)\n'))
+                rst.append(_("\n(use 'hg help' for the full list "
+                             "of commands)\n"))
         else:
             if name == 'shortlist':
-                rst.append(_('\n(use "hg help" for the full list of commands '
-                             'or "hg -v" for details)\n'))
+                rst.append(_("\n(use 'hg help' for the full list of commands "
+                             "or 'hg -v' for details)\n"))
             elif name and not full:
-                rst.append(_('\n(use "hg help %s" to show the full help '
-                             'text)\n') % name)
+                rst.append(_("\n(use 'hg help %s' to show the full help "
+                             "text)\n") % name)
             elif name and cmds and name in cmds.keys():
-                rst.append(_('\n(use "hg help -v -e %s" to show built-in '
-                             'aliases and global options)\n') % name)
+                rst.append(_("\n(use 'hg help -v -e %s' to show built-in "
+                             "aliases and global options)\n") % name)
             else:
-                rst.append(_('\n(use "hg help -v%s" to show built-in aliases '
-                             'and global options)\n')
+                rst.append(_("\n(use 'hg help -v%s' to show built-in aliases "
+                             "and global options)\n")
                            % (name and " " + name or ""))
         return rst
 
@@ -489,8 +498,8 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
 
         try:
             cmdutil.findcmd(name, commands.table)
-            rst.append(_('\nuse "hg help -c %s" to see help for '
-                       'the %s command\n') % (name, name))
+            rst.append(_("\nuse 'hg help -c %s' to see help for "
+                       "the %s command\n") % (name, name))
         except error.UnknownCommand:
             pass
         return rst
@@ -527,8 +536,8 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
             modcmds = set([c.partition('|')[0] for c in ct])
             rst.extend(helplist(modcmds.__contains__))
         else:
-            rst.append(_('(use "hg help extensions" for information on enabling'
-                       ' extensions)\n'))
+            rst.append(_("(use 'hg help extensions' for information on enabling"
+                       " extensions)\n"))
         return rst
 
     def helpextcmd(name, subtopic=None):
@@ -540,8 +549,8 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
                               "extension:") % cmd, {ext: doc}, indent=4,
                        showdeprecated=True)
         rst.append('\n')
-        rst.append(_('(use "hg help extensions" for information on enabling '
-                   'extensions)\n'))
+        rst.append(_("(use 'hg help extensions' for information on enabling "
+                   "extensions)\n"))
         return rst
 
 
@@ -566,7 +575,7 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
                 rst.append('\n')
         if not rst:
             msg = _('no matches')
-            hint = _('try "hg help" for a list of topics')
+            hint = _("try 'hg help' for a list of topics")
             raise error.Abort(msg, hint=hint)
     elif name and name != 'shortlist':
         queries = []
@@ -589,7 +598,7 @@ def help_(ui, name, unknowncmd=False, full=True, subtopic=None, **opts):
                 raise error.UnknownCommand(name)
             else:
                 msg = _('no such help topic: %s') % name
-                hint = _('try "hg help --keyword %s"') % name
+                hint = _("try 'hg help --keyword %s'") % name
                 raise error.Abort(msg, hint=hint)
     else:
         # program name
