@@ -7,6 +7,8 @@
 
 from __future__ import absolute_import
 
+import hashlib
+
 from .i18n import _
 from . import (
     bdiff,
@@ -25,15 +27,16 @@ def _findexactmatches(repo, added, removed):
     # Get hashes of removed files.
     hashes = {}
     for i, fctx in enumerate(removed):
-        repo.ui.progress(_('searching for exact renames'), i, total=numfiles)
-        h = util.sha1(fctx.data()).digest()
+        repo.ui.progress(_('searching for exact renames'), i, total=numfiles,
+                         unit=_('files'))
+        h = hashlib.sha1(fctx.data()).digest()
         hashes[h] = fctx
 
     # For each added file, see if it corresponds to a removed file.
     for i, fctx in enumerate(added):
         repo.ui.progress(_('searching for exact renames'), i + len(removed),
-                total=numfiles)
-        h = util.sha1(fctx.data()).digest()
+                total=numfiles, unit=_('files'))
+        h = hashlib.sha1(fctx.data()).digest()
         if h in hashes:
             yield (hashes[h], fctx)
 
@@ -49,7 +52,7 @@ def _findsimilarmatches(repo, added, removed, threshold):
     copies = {}
     for i, r in enumerate(removed):
         repo.ui.progress(_('searching for similar files'), i,
-                         total=len(removed))
+                         total=len(removed), unit=_('files'))
 
         # lazily load text
         @util.cachefunc
@@ -105,4 +108,3 @@ def findrenames(repo, added, removed, threshold):
         for (a, b, score) in _findsimilarmatches(repo,
                 sorted(addedfiles), sorted(removedfiles), threshold):
             yield (a.path(), b.path(), score)
-
