@@ -10,6 +10,12 @@ import sys
 # write static check patterns here
 perfpypats = [
   [
+    (r'(branchmap|repoview)\.subsettable',
+     "use getbranchmapsubsettable() for early Mercurial"),
+    (r'\.(vfs|svfs|opener|sopener)',
+     "use getvfs()/getsvfs() for early Mercurial"),
+    (r'ui\.configint',
+     "use getint() instead of ui.configint() for early Mercurial"),
   ],
   # warnings
   [
