@@ -1,6 +1,13 @@
+from __future__ import absolute_import, print_function
+
 import os
-from hgext import color
-from mercurial import dispatch, ui
+from hgext import (
+    color,
+)
+from mercurial import (
+    dispatch,
+    ui as uimod,
+)
 
 # ensure errors aren't buffered
 testui = color.colorui()
@@ -8,7 +15,7 @@ testui.pushbuffer()
 testui.write(('buffered\n'))
 testui.warn(('warning\n'))
 testui.write_err('error\n')
-print repr(testui.popbuffer())
+print(repr(testui.popbuffer()))
 
 # test dispatch.dispatch with the same ui object
 hgrc = open(os.environ["HGRCPATH"], 'w')
@@ -16,7 +23,7 @@ hgrc.write('[extensions]\n')
 hgrc.write('color=\n')
 hgrc.close()
 
-ui_ = ui.ui()
+ui_ = uimod.ui()
 ui_.setconfig('ui', 'formatted', 'True')
 
 # we're not interested in the output, so write that to devnull
@@ -28,6 +35,6 @@ def runcmd():
     dispatch.dispatch(dispatch.request(['version', '-q'], ui_))
 
 runcmd()
-print "colored? " + str(issubclass(ui_.__class__, color.colorui))
+print("colored? " + str(issubclass(ui_.__class__, color.colorui)))
 runcmd()
-print "colored? " + str(issubclass(ui_.__class__, color.colorui))
+print("colored? " + str(issubclass(ui_.__class__, color.colorui)))
